package ap;

import ap.parameters.{GoalSettings, Param}
import ap.parser.PlainTranslator
import ap.terfor.Formula
import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.proof.{ModelSearchProver, ExhaustiveProver, ConstraintSimplifier,
                 ProofTree, Timeout}
import ap.proof.goal.{Goal, SymbolWeights}

abstract class AbstractFileProver(reader : java.io.Reader, output : Boolean,
                                  timeout : Int, userDefStoppingCond : => Boolean,
                                  initialGoalSettings : GoalSettings) {

  protected def println(str : => String) : Unit = (if (output) Predef.println(str))
  
  val (formula, eliminatedConstants, order) = {
    val input = PlainTranslator(reader)
    reader.close
    input
  }

  private val goalSettings =
    Param.SYMBOL_WEIGHTS.set(initialGoalSettings,
                             SymbolWeights.normSymbolFrequencies(formula, 1000))
  
  protected def findCounterModel : Conjunction = {
    println("Constructing countermodel ...")
    val timeBefore = System.currentTimeMillis
    val stoppingCond = () => {
      if ((System.currentTimeMillis - timeBefore > timeout) || userDefStoppingCond)
        Timeout.raise
    }

    Timeout.withChecker(stoppingCond) {
      ModelSearchProver(formula, order, goalSettings)
    }
  }
  
  protected def findModel(f : Conjunction) : Conjunction =
    ModelSearchProver(f.negate, f.order)
  
  protected def constructProofTree(simplifier : ConstraintSimplifier) : ProofTree = {
    // explicitly quantify all universal variables
    
    val closedFor =
      Conjunction.quantify(Quantifier.ALL, eliminatedConstants, formula, order)
    
    println("Proving ...")
    
    val timeBefore = System.currentTimeMillis
    val stoppingCond = () => {
      if ((System.currentTimeMillis - timeBefore > timeout) ||
          userDefStoppingCond)
        Timeout.raise
    }

    Timeout.withChecker(stoppingCond) {
      new ExhaustiveProver(simplifier, true, goalSettings)(closedFor, order)
    }
  }

}
